@@ -1,88 +1,83 @@
-# -*- encoding: utf8 -*-
-from __future__ import unicode_literals
-
 import re
-from collections import namedtuple, OrderedDict
 from datetime import datetime
+from io import BytesIO
 from logging import getLogger
+from typing import NamedTuple, List, Union, Optional
 from xml.etree import ElementTree
-from zipfile import ZipFile
+from zipfile import ZipFile, BadZipFile
 
 from dbf_light import Dbf
 
 from .exceptions import PycbrfException
-from .utils import string_types, BytesIO, WithRequests, text_type
+from .utils import WithRequests
 
 LOG = getLogger(__name__)
 
 
-BankLegacy = namedtuple(
-    'BankLegacy',
-    [
-        'bic',
-        'name',
-        'name_full',
-        'region_code',
-        'region',
-        'zip',
-        'place_type',
-        'place',
-        'address',
-        'rkc_bic',
-        'term',
-        'date_added',
-        'date_updated',
-        'date_change',
-        'mfo',
-        'corr',
-        'corr_bik',
-        'phone',
-        'telegraph',
-        'commutator',
-        'okpo',
-        'regnum',
-        'type',
-        'pay_type',
-        'control_code',
-        'control_date',
-        'swift',
-    ]
-)
-"""Represents bank entry in legacy format.
-
-Such objects will populate Banks().banks
-
-"""
-
-Bank = namedtuple(
-    'Bank',
-    [
-        'bic',
-        'name_full',
-        'name_full_eng',
-        'region_code',
-        'country_code',
-        'zip',
-        'place_type',
-        'place',
-        'address',
-        'date_added',
-        'corr',
-        'regnum',
-        'type',
-        'swift',
-        'restricted',
-        'restrictions',
-    ]
-)
-"""Represents bank entry in current format.
-
-Such objects will populate Banks().banks
-
-"""
+class _BankBase(NamedTuple):
+    """Base from banks."""
 
 
-class Restriction(object):
+class BankLegacy(_BankBase, NamedTuple):
+    """Represents bank entry in legacy format.
+
+    Such objects will populate Banks().banks
+
+    """
+    bic: str
+    name: str
+    name_full: str
+    region_code: str
+    region: NamedTuple
+    zip: str
+    place_type: NamedTuple
+    place: str
+    address: str
+    rkc_bic: str
+    term: int
+    date_added: datetime.date
+    date_updated: Optional[datetime.date]
+    date_change: Optional[datetime.date]
+    mfo: str
+    corr: str
+    corr_bik: str
+    phone: str
+    telegraph: str
+    commutator: str
+    okpo: str
+    regnum: str
+    type: NamedTuple
+    pay_type: NamedTuple
+    control_code: str
+    control_date: Optional[datetime.date]
+    swift: Optional[str]
+
+
+class Bank(_BankBase, NamedTuple):
+    """Represents bank entry in current format.
+
+    Such objects will populate Banks().banks
+
+    """
+    bic: str
+    name_full: str
+    name_full_eng: str
+    region_code: str
+    country_code: str
+    zip: str
+    place_type: str
+    place: str
+    address: str
+    date_added: datetime
+    corr: str
+    regnum: str
+    type: str
+    swift: str
+    restricted: bool
+    restrictions: List['Restriction']
+
+
+class Restriction:
     """Represents a restriction imposed on an institution."""
 
     __slots__ = ['code', 'date', 'account', 'title']
@@ -101,7 +96,7 @@ class Restriction(object):
 
     """
 
-    def __init__(self, code, date, account=''):
+    def __init__(self, code: str, date: datetime.date, account: str = ''):
         self.code = code
 
         self.date = date
@@ -112,14 +107,12 @@ class Restriction(object):
         self.title = self.codes.get(code, '')
 
     def __str__(self):
-        return '%s %s [%s] %s' % (
-            self.date, self.code, self.account, self.title,
-        )
+        return f'{self.date} {self.code} [{self.account}] {self.title}'
 
 
 class Banks(WithRequests):
 
-    def __init__(self, on_date=None):
+    def __init__(self, on_date: Union[datetime, str] = None):
         """Fetches BIC data.
 
         :param datetime|str on_date: Date to get data for.
@@ -127,81 +120,74 @@ class Banks(WithRequests):
             If not set data for today will be fetched.
 
         """
-        if isinstance(on_date, string_types):
+        if isinstance(on_date, str):
             on_date = datetime.strptime(on_date, '%Y-%m-%d')
 
         on_date = on_date or datetime.now()
         legacy = on_date < datetime(2018, 7, 1)
 
-        self.banks = self._get_data(on_date=on_date, legacy=legacy)
+        self.banks: Union[List['Bank'], List['BankLegacy']] = self._get_data(on_date=on_date, legacy=legacy)
         self.on_date = on_date
         self.legacy = legacy  # CB RF radically changed format from DBF (legacy) to XML.
 
-    def __getitem__(self, item):
-        """
-        :param str item:
-        :rtype: Bank
-        """
+    def __getitem__(self, item: str) -> Optional[Union['Bank', 'BankLegacy']]:
+
         key = 'swift' if len(item) in {8, 11} else 'bic'
         indexed = {getattr(bank, key): bank for bank in self.banks}
 
-        return indexed[item]
+        return indexed.get(item)
 
     @classmethod
-    def get_titles(cls):
-        """Returns fields titles.
+    def get_titles(cls) -> dict:
+        """Returns fields titles."""
 
-        :rtype: OrderedDict
-        """
-        titles = OrderedDict((
-            ('bic', 'БИК'),  # Банковский идентификационный код
-            ('swift', 'Код SWIFT'),
-            ('name', 'Название'),
-            ('name_full', 'Полное название'),
-            ('name_full_eng', 'Полное название (англ.)'),
+        titles = {
+            'bic': 'БИК',  # Банковский идентификационный код
+            'swift': 'Код SWIFT',
+            'name': 'Название',
+            'name_full': 'Полное название',
+            'name_full_eng': 'Полное название (англ.)',
 
-            ('date_added', 'Дата добавления записи'),
-            ('date_updated', 'Дата обновления записи'),
-            ('date_change', 'Дата изменения реквизитов'),
+            'date_added': 'Дата добавления записи',
+            'date_updated': 'Дата обновления записи',
+            'date_change': 'Дата изменения реквизитов',
 
-            ('restricted', 'С ограничениями'),  # Fuzzy analogy for `control_code`.
-            ('restrictions', 'Ограничения'),
-            ('control_code', 'Код контроля'),
-            ('control_date', 'Дата контроля'),
+            'restricted': 'С ограничениями',  # Fuzzy analogy for `control_code`.
+            'restrictions': 'Ограничения',
+            'control_code': 'Код контроля',
+            'control_date': 'Дата контроля',
 
-            ('corr', 'Кор. счёт'),
-            ('corr_bik', 'Кор. счёт (расчёты с БИК)'),
+            'corr': 'Кор. счёт',
+            'corr_bik': 'Кор. счёт (расчёты с БИК)',
 
-            ('regnum', 'Регистрационный номер'),
-            ('mfo', 'Номер МФО'),
-            ('okpo', 'Номер ОКПО'),  # Классификатор предприятий и организаций
-            ('type', 'Тип'),
-            ('pay_type', 'Тип расчётов'),
+            'regnum': 'Регистрационный номер',
+            'mfo': 'Номер МФО',
+            'okpo': 'Номер ОКПО',  # Классификатор предприятий и организаций
+            'type': 'Тип',
+            'pay_type': 'Тип расчётов',
 
-            ('country_code', 'Код страны'),
-            ('region_code', 'Код региона ОКАТО'),  # Классификатор объектов административно-территориального деления
-            ('region', 'Регион'),
-            ('zip', 'Индекс'),
-            ('place_type', 'Тип населённого пункта'),
-            ('place', 'Населённый пункт'),
-            ('address', 'Адрес'),
+            'country_code': 'Код страны',
+            'region_code': 'Код региона ОКАТО',  # Классификатор объектов административно-территориального деления
+            'region': 'Регион',
+            'zip': 'Индекс',
+            'place_type': 'Тип населённого пункта',
+            'place': 'Населённый пункт',
+            'address': 'Адрес',
 
-            ('phone', 'Телефон'),
-            ('telegraph', 'Телеграф'),
-            ('commutator', 'Коммутатор'),
+            'phone': 'Телефон',
+            'telegraph': 'Телеграф',
+            'commutator': 'Коммутатор',
 
-            ('rkc_bic', 'БИК РКЦ'),  # Рассчётно-кассовый центр
-            ('term', 'Срок проведения расчётов (дней)'),
-        ))
+            'rkc_bic': 'БИК РКЦ',  # Рассчётно-кассовый центр
+            'term': 'Срок проведения расчётов (дней)',
+        }
         return titles
 
     @classmethod
-    def annotate(cls, banks):
+    def annotate(cls, banks: List['Bank']) -> List[dict]:
         """Annotates bank objects with titles.
 
-        :param list[Bank] banks: A list of Bank objects to annotate.
-
-        :rtype: list
+        :param banks: A list of Bank objects to annotate.
 
         """
         titles = cls.get_titles()
@@ -217,7 +203,11 @@ class Banks(WithRequests):
         unset = object()
 
         for bank in banks:
-            bank_dict = OrderedDict()
+
+            if not bank:
+                continue
+
+            bank_dict = {}
             bank = bank._asdict()
 
             for alias, title in titles.items():
@@ -227,14 +217,14 @@ class Banks(WithRequests):
                     # Some fields may be missing in Bank/BankLegacy
                     continue
 
-                if isinstance(value, tuple):
+                if isinstance(value, _BankBase):
                     value = pick_value(value._asdict())
 
                 elif isinstance(value, bool):
                     value = 'Да' if value else 'Нет'
 
                 elif isinstance(value, list):
-                    value = '\n  ' + '\n  '.join(map(text_type, value))
+                    value = '\n  ' + '\n  '.join(map(str, value))
 
                 bank_dict[title] = value or ''
 
@@ -243,14 +233,19 @@ class Banks(WithRequests):
         return annotated
 
     @classmethod
-    def _get_archive(cls, url):
-        LOG.debug('Fetching data from %s ...', url)
+    def _get_archive(cls, url: str) -> Optional[BytesIO]:
+        LOG.debug(f'Fetching data from {url} ...')
 
         response = cls._get_response(url, stream=True)
+
+        if response.status_code != 200:
+            # E.g. 404 is expected on weekends.
+            return None
+
         return BytesIO(response.content)
 
     @classmethod
-    def _read_zipped_xml(cls, zipped):
+    def _read_zipped_xml(cls, zipped: BytesIO) -> ElementTree:
 
         with ZipFile(zipped, 'r') as zip_:
             filename = zip_.namelist()[0]
@@ -259,7 +254,7 @@ class Banks(WithRequests):
                 return ElementTree.fromstring(f.read())
 
     @classmethod
-    def _get_data(cls, on_date, legacy=False):
+    def _get_data(cls, on_date: datetime, legacy: bool = False) -> Union[List['Bank'], List['BankLegacy']]:
 
         if legacy:
             return cls._get_data_dbf(on_date=on_date)
@@ -267,18 +262,20 @@ class Banks(WithRequests):
         return cls._get_data_xml(on_date=on_date)
 
     @classmethod
-    def _get_data_xml(cls, on_date):
+    def _get_data_xml(cls, on_date: datetime) -> List['Bank']:
         """Справочник БИК (Клиентов Банка России). XML ED807
 
         http://www.cbr.ru/development/formats/
 
-        :param datetime on_date:
-        :rtype: list
+        :param on_date:
 
         """
-        url = 'http://www.cbr.ru/VFS/mcirabis/BIKNew/%sED01OSBR.zip' % on_date.strftime('%Y%m%d')
+        data = cls._get_archive(f"http://www.cbr.ru/VFS/mcirabis/BIKNew/{on_date.strftime('%Y%m%d')}ED01OSBR.zip")
 
-        xml = cls._read_zipped_xml(cls._get_archive(url))
+        if data is None:
+            return []
+
+        xml = cls._read_zipped_xml(data)
 
         def parse_date(val):
             if not val:
@@ -313,18 +310,18 @@ class Banks(WithRequests):
 
         banks = []
 
-        for entry in xml.findall(ns + 'BICDirectoryEntry'):
+        for entry in xml.findall(f'{ns}BICDirectoryEntry'):
             restrictions_applied = []
 
             bic = entry.attrib['BIC']
 
-            el_info = entry.find(ns + 'ParticipantInfo')
+            el_info = entry.find(f'{ns}ParticipantInfo')
             attrs_info = el_info.attrib
 
             if attrs_info['ParticipantStatus'] == 'PSDL':  # Маркер удаления
                 continue
 
-            for el_restriction in el_info.findall(ns + 'RstrList'):
+            for el_restriction in el_info.findall(f'{ns}RstrList'):
                 attrs = el_restriction.attrib
                 code = attrs['Rstr']
                 restrictions_applied.append(Restriction(
@@ -334,13 +331,13 @@ class Banks(WithRequests):
 
             swiftcode = None
 
-            for el_swift in entry.findall(ns + 'SWBICS'):
+            for el_swift in entry.findall(f'{ns}SWBICS'):
                 if el_swift.attrib.get('DefaultSWBIC'):
                     swiftcode = el_swift.attrib['SWBIC']  # [8/11]
                     break
 
             corr = ''
-            accounts = entry.findall(ns + 'Accounts')
+            accounts = entry.findall(f'{ns}Accounts')
 
             for el_account in accounts:
                 """
@@ -370,7 +367,7 @@ class Banks(WithRequests):
                 assert corr == '', 'More than one correspondent account detected'
                 corr = attrs['Account']  # [20]
 
-                for el_restriction in el_account.findall(ns + 'AccRstrList'):
+                for el_restriction in el_account.findall(f'{ns}AccRstrList'):
                     attrs = el_restriction.attrib
                     code = attrs['AccRstr']
                     restrictions_applied.append(Restriction(
@@ -401,30 +398,28 @@ class Banks(WithRequests):
         return banks
 
     @classmethod
-    def _read_zipped_db(cls, zipped, filename):
+    def _read_zipped_db(cls, zipped: BytesIO, filename: str):
         with Dbf.open_zip(filename, zipped, case_sensitive=False) as dbf:
             for row in dbf:
                 yield row
 
     @classmethod
-    def _get_data_dbf(cls, on_date):
-        """
+    def _get_data_dbf(cls, on_date: datetime) -> List['BankLegacy']:
 
-        :param datetime on_date:
-        :rtype: list
-
-        """
         try:
             swifts = cls._get_data_swift()
 
         except PycbrfException:
             swifts = {}
 
-        url = 'http://www.cbr.ru/vfs/mcirabis/BIK/bik_db_%s.zip' % on_date.strftime('%d%m%Y')
-        zip = cls._get_archive(url)
+        zipped = cls._get_archive(
+            f"http://www.cbr.ru/vfs/mcirabis/BIK/bik_db_{on_date.strftime('%d%m%Y')}.zip")
+
+        if zipped is None:
+            return []
 
         def get_indexed(dbname, index):
-            return {getattr(region, index): region for region in cls._read_zipped_db(zip, filename=dbname)}
+            return {getattr(region, index): region for region in cls._read_zipped_db(zipped, filename=dbname)}
 
         regions = get_indexed('reg.dbf', 'rgn')
         types = get_indexed('pzn.dbf', 'pzn')
@@ -433,7 +428,7 @@ class Banks(WithRequests):
 
         banks = []
 
-        for row in cls._read_zipped_db(zip, filename='bnkseek.dbf'):
+        for row in cls._read_zipped_db(zipped, filename='bnkseek.dbf'):
             region_code = row.rgn
             bic = row.newnum
 
@@ -491,12 +486,12 @@ class Banks(WithRequests):
         return banks
 
     @classmethod
-    def _get_data_swift(cls):
+    def _get_data_swift(cls) -> dict:
         # At some moment static URL has became dynamic, and now ne need to search for it every time.
         host = 'http://www.cbr.ru'
-        response = cls._get_response('%s/analytics/digest/' % host)
+        response = cls._get_response(f'{host}/analytics/digest/')
 
-        found = re.findall('href="([^."]+\.zip)"', response.text)
+        found = re.findall(r'href="([^."]+\.zip)"', response.text)
 
         if not found or len(found) > 1:
             raise PycbrfException('Unable to get SWIFT info archive link')
